@@ -8,6 +8,7 @@ import anyio
 import logging
 import time
 import sys
+import os
 from contextlib import asynccontextmanager, suppress, nullcontext, aclosing
 from traceback import format_exception
 from platform import uname
@@ -16,9 +17,9 @@ from functools import partial
 import outcome
 from attrs import define,field
 
-from mqttproto import RetainHandling
+from mqttproto import RetainHandling, QoS
 from moat.lib.cmd.base import MsgSender, Caller
-from moat.util import CtxObj, P, Root, ValueEvent, timed_ctx, gen_ident, ungroup, NotGiven,Path, PathLongener,srepr
+from moat.util import CtxObj, P, Root, ValueEvent, timed_ctx, gen_ident, ungroup, NotGiven,Path, PathLongener,srepr, attrdict
 from moat.util.compat import CancelledError
 from moat.util.random import al_unique
 
@@ -113,10 +114,11 @@ class LinkCommon(CmdCommon):
 
     def __init__(self, cfg, name: str | None = None, is_server:bool = False):
         self.cfg = cfg
+        self._id = gen_ident(12, alphabet=al_unique)
         if name is None:
             name = cfg.get("client_id")
         if name is None:
-            name = "_" + gen_ident(10, alphabet=al_unique)
+            name = "_" + self._id
         self.name = name
         self.is_server = is_server
 
@@ -134,6 +136,10 @@ class LinkCommon(CmdCommon):
 
         return super().handle(msg, rpath, *add)
 
+    @property
+    def id(self):
+        return self._id
+
     async def _connected_port(self, *, task_status=anyio.TASK_STATUS_IGNORED):
         async with self._connect_one(self._port) as hdl:
             task_status.started(hdl)
@@ -216,6 +222,18 @@ class _Sender(MsgSender):
     def root(self):
         return self
 
+    @property
+    def name(self) -> str:
+        return self._link.name
+
+    @property
+    def cfg(self) -> attrdict:
+        return self._link.cfg
+
+    @property
+    def id(self) -> str:
+        return self._link.id
+
     async def handle(self, msg: Msg, rcmd: list) -> Awaitable[None]:
         """
         Standard handler, forwards to the remote side.
@@ -483,14 +501,24 @@ class Link(LinkCommon, CtxObj):
     _last_link: Msg | None = None
     _last_link_seen: anyio.Event
     _port: str|None=None
+    _state: str="init"
 
     def __init__(self, cfg, name: str | None = None):
         super().__init__(cfg, name=name)
         self._retry_msgs: set[BasicCmd] = set()
         self._server_up = anyio.Event()
+        self._state_change = anyio.Event()
         with suppress(AttributeError):
             self._port = self.cfg.client.port
 
+    async def set_state(self, state:str):
+        """
+        Set the state string for our ping message
+        """
+        if state != self._state:
+            self._state = state
+            self._state_change.set()
+
     async def get_link(self):
         """
         This method refreshes the link to the server if it happens to be down.
@@ -499,17 +527,52 @@ class Link(LinkCommon, CtxObj):
             await self._server_up.wait()
         return self.current_server
 
+    @property
+    def _ping_path(self):
+        return P("run.ping.id")/self._id
+
+    async def _ping(self, *, task_status=anyio.TASK_STATUS_IGNORED):
+        path = Root.get()+self._ping_path
+        while True:
+            await self.backend.send(path, data=dict(
+                up=True,
+                state=self._state,
+            ), retain=False, meta=False)
+
+            if task_status is not anyio.TASK_STATUS_IGNORED:
+                await self.sdr.d_set(P("run.id")/self.id,data=dict(
+                    host=uname().node,
+                    pid=os.getpid(),
+                ), retain=True)
+                task_status.started()
+                task_status=anyio.TASK_STATUS_IGNORED
+
+            if self._state == "init":
+                self._state = "auto"
+
+            with anyio.move_on_after(self.cfg.timeout.ping):
+                await self._state_change.wait()
+                self._state_change = anyio.Event()
+
     @asynccontextmanager
     async def _ctx(self):
         from .backend import get_backend
 
-        async with (
-            get_backend(self.cfg, name=self.name) as backend,
-            anyio.create_task_group() as self.tg,
-        ):
-            self.backend = backend
-            token = Root.set(self.cfg["root"])
-            try:
+        token = Root.set(self.cfg["root"])
+        try:
+            will=attrdict(
+                data=dict(
+                    up=False,
+                    state="disconnected",
+                    ),
+                topic=P(":R.run.ping.id")/self.id,
+                retain=False,
+                qos=QoS.AT_LEAST_ONCE,
+            )
+            async with (
+                get_backend(self.cfg, name=self.name, will=will) as self.backend,
+                anyio.create_task_group() as self.tg,
+            ):
                 if self._port is not None:
                     sdr = await self.tg.start(self._connected_port)
                 else:
@@ -521,11 +584,16 @@ class Link(LinkCommon, CtxObj):
                 sdr.add_sub("d")
                 sdr.add_sub("e")
                 sdr.add_sub("i")
-                yield sdr
+                try:
+                    self.sdr = sdr
+                    await self.tg.start(self._ping)
+                    yield sdr
+                finally:
+                    del self.sdr
                 self.tg.cancel_scope.cancel()
-            finally:
-                Root.reset(token)
-            return
+                await self.backend.send(Root.get()+self._ping_path, data=dict(up=False,state="closed"), retain=False, meta=False)
+        finally:
+            Root.reset(token)
 
     def cancel(self):
         "Stop me"
@@ -657,6 +725,8 @@ class Link(LinkCommon, CtxObj):
                     self.cfg.client.init_timeout, self._connect_one(remote, srv.data)
                 ) as rem:
                     await self._connect_run(rem, task_status=task_status)
+            except EnvironmentError as exc:
+                self.logger.warning("Link failed: %r", remote)
             except Exception as exc:
                 self.logger.warning("Link failed: %r", remote, exc_info=exc)
 
@@ -687,7 +757,7 @@ class BasicLink(LinkCommon, CtxObj):
             except Exception as exc:
                 if yielded:
                     raise
-                self.logger.warning("Link failed: %r", remote, exc_info=exc)
+                self.logger.warning("Link failed: %r", remote, exc_info=None if isinstance(exc,EnvironmentError) else exc)
                 if err is None:
                     err = exc
             else:
@@ -755,7 +825,7 @@ class Watcher(CtxObj):
 
     async def _updates(self, *, task_status):
         "get updates from MQTT"
-        plen = 1+len(self.path)
+        plen = 1+len(self.path)  # add the root tag
         async with self.link.monitor(Root.get()+self.path, subtree=self.subtree, retained=False) as mon:
             task_status.started()
             async for msg in mon:
