@@ -43,7 +43,7 @@ async def cli(ctx):
     obj = ctx.obj
     cfg = obj.cfg["link"]
     if not obj.name:
-        obj.name = "x_"+gen_ident()
+        obj.name = "mon_"+gen_ident()
 
     set_root(cfg)
     obj.conn = await ctx.with_async_resource(get_backend(cfg, name=obj.name))
